include "Vram.inc"
include "SharedRam.inc"
include "Share.inc"
include "VVram.inc"
include "Chars.inc"

ext HaltSubsys, ResumeSubsys
ext MonoPattern, ColorPattern
ext VVramFront_, RowFlags_

dseg
Backup:
    defs VVramWidth*VVramHeight
PatternRam: public PatternRam
    defs PatternSize*Char_End


zseg
CharColor:
    defb 0
CharCount:
    defb 0
pCount:
    defb 0    
yCount:
    defb 0
xCount:
byteCount:
    defb 0
flagByte:
bitCount:
    defb 0
colorBits:
    defb 0
maskByte:
invertedByte:
    defb 0
pVram:
    defw 0
pFlags:
    defw 0


cseg
ColorTable:
defb 64, 7 ; Ascii
defb 16, 5 ; Logo
defb 2, 2 ; Wall
defb 1, 7 ; HardWall
defb 4, 6 ; Meter
defb 2, 1 ; Fence
defb 0
InitVram: public InitVram
    pshs u,cc | orcc #$50
        ldy #MonoPattern
        ldx #PatternRam
        ldu #ColorTable
        do
            ldb 0,u
        while ne
            leau 1,u
            lda 0,u
            leau 1,u
            bsr MakePatternMono
        wend
        ldy #ColorPattern
        ldx #PatternRam+PatternSize*Char_MyRobo
        ldb #Char_End-Char_MyRobo
        bsr MakePatternColor
    puls u,cc
rts

MakePatternMono:
    std <CharColor
    orcc #$50
    do
        lda <CharColor | sta <colorBits
        lda #PlaneCount | sta <pCount
        do
            pshs y
                lda #CharHeight | sta <yCount
                do
                    clrb
                    lda colorBits
                    bita #$01
                    if ne
                        orb ,y
                    endif
                    bita #$10
                    if ne
                        lda ,y
                        coma
                        sta <invertedByte
                        orb <invertedByte
                    endif
                    lda #VramStep | sta <byteCount
                    do
                        lda #8/VramStep | sta <bitCount
                        clra
                        do
                            lsla
                            lsla
                            lslb
                            if cs
                                ora #$03
                            endif
                            dec <bitCount
                        while ne | wend
                        sta ,x+
                        dec <byteCount
                    while ne | wend
                    leay 1,y
                    dec <yCount
                while ne | wend
            puls y
            lsr <colorBits
            dec <pCount
        while ne | wend
        leay CharHeight,y
        dec <CharCount
    while ne | wend
    andcc #not $50
rts

MakePatternColor:
    stb <CharCount
    orcc #$50

    do
        lda #PatternSize/2
        sta <yCount
        do
            ldb ,y+
            lda #2
            sta <byteCount
            do
                lda #4
                sta <bitCount
                clra
                do
                    lsla
                    lsla
                    lslb
                    if cs
                        ora #$03
                    endif
                    dec <bitCount
                while ne | wend
                sta ,x+
                dec <byteCount
            while ne | wend
            dec <yCount
        while ne | wend
        dec CharCount
    while ne | wend
    andcc #not $50
rts


cseg
ClearRowFlags:
    pshs x
        ldx #RowFlags_
        do
            clr ,x+
            cmpx #RowFlags_+4
        while ne | wend
    puls x
rts

; void ClearScreen();
cseg
ClearScreen_:   public ClearScreen_
    pshs x,b,a
        jsr HaltSubsys
        lda #SubCommand_Clear
        sta SharedRam+Share_Command
        jsr ResumeSubsys
        clr lastChar

        ldx #Backup
        ldd #VVramWidth*VVramHeight
        do
            clr ,x+
            subd #1
        while ne | wend

        bsr ClearRowFlags
    puls x,b,a
rts


; word Put(word vram, byte c);
dseg
    lastChar: defb 0
cseg
Put_: public Put_
    pshs a,b,x,y
        jsr HaltSubsys
        lda #SubCommand_PutP | sta SharedRam+Share_Command
        stx SharedRam+Share_Address
        cmpb lastChar
        if ne
            stb lastChar
            pshs cc | orcc #$40
                lda #PatternSize
                mul
                addd #PatternRam
                tfr d,x
                pshs cc | orcc #$40
                    ldy #SharedRam+Share_Pattern
                    ldb #PatternSize
                    do
                        lda ,x+
                        sta ,y+
                        decb
                    while ne | wend
                puls cc
            puls cc
        endif
        jsr ResumeSubsys
    puls a,b,x,y
    tfr x,d
    addd #2
rts


VVramToVramRow: public VVramToVramRow
    lda #VVramWidth
    sta <xCount
    do 
        ldb ,x+
        cmpb ,y
        if ne
            stb ,y
            pshs x
                ldx <pVram
                bsr Put_
            puls x
        endif
        leay 1,y
        pshs x
            ldx <pVram
            leax 2,x
            stx <pVram
        puls x
        dec <xCount
    while ne | wend
    pshs x
        ldx <pVram
        leax VramRowSize-VVramWidth*2,x
        stx <pVram
    puls x
rts


; void VVramToVramAll();
VVramToVramAll_: public VVramToVramAll_
    pshs a,b,x,y
        ldx #Vram+VramRowSize
        stx <pVram
        ldx #VVramFront_
        ldy #Backup
        lda #VVramHeight
        sta <yCount
        do
            bsr VVramToVramRow
            dec <yCount
        while ne | wend
        bsr ClearRowFlags
    puls a,b,x,y
rts


; void VVramToVramChanged();
VVramToVramChanged_: public VVramToVramChanged_
    pshs a,b,x,y
        ldx #Vram+VramRowSize
        stx <pVram
        ldx #RowFlags_
        lda ,x+
        sta <flagByte
        stx <pFlags
        lda #1
        sta <maskByte

        ldx #VVramFront_
        ldy #Backup
        lda #VVramHeight
        sta <yCount
        do
            lda <flagByte
            anda <maskByte
            if ne
                bsr VVramToVramRow
            else
                leax VVramWidth,x
                leay VVramWidth,y
                pshs x
                    ldx <pVram
                    leax VramRowSize,x
                    stx <pVram
                puls x
            endif
            lsl <maskByte
            if eq
                pshs x
                    ldx <pFlags
                    lda ,x+
                    sta <flagByte
                    stx <pFlags
                    lda #1
                    sta <maskByte
                puls x
            endif
            dec <yCount
        while ne | wend
        bsr ClearRowFlags
    puls a,b,x,y
rts